// ----------------------------------------------------------
// This Source Code Form is subject to the terms of the
// Mozilla Public License, v.2.0. If a copy of the MPL
// was not distributed with this file, You can obtain one
// at http://mozilla.org/MPL/2.0/.
// ----------------------------------------------------------
// Codebase: https://github.com/ArKuznetsov/yard/
// ----------------------------------------------------------

Перем МенеджерОбработкиДанных;         // Объект            - обработчик-менеджер, вызвавшая данный обработчик
Перем Идентификатор;                   // Строка            - идентификатор обработчика,
                                       //                     заданный обработкой-менеджером
Перем ПараметрыОбработки;              // Структура         - параметры обработки
Перем Лог;                             // Объект            - объект записи лога приложения

Перем ПутьККаталогуДистрибутивов;      // Строка            - путь к каталогу содержащему дистрибутивы конфигурации
                                       //                     для распаковки
Перем КаталогДляРаспаковкиШаблонов;    // Строка            - каталог для распаковки шаблонов конфигурации
Перем КаталогВАрхивеДляРаспаковкиEFD;  // Строка            - регулярное выражение, отбора каталогов
                                       //                     в архиве EFD для распаковки
Перем ФайлыДляРаспаковкиEFD;           // Массив(Строка)    - список файлов для распаковки из архива EFD дистрибутива
                                       //                     конфигурации, если не указан,
                                       //                     то распаковываются все файлы
Перем РаспаковыватьСуществующие;       // Булево            - Истина - будут распакованы все найденные дистрибутивы
                                       //                     независимо от существующих в каталоге для распаковки
                                       //                     Ложь - будут оаспакованы только отсутствующие
                                       //                     в каталоге для распаковки дистрибутивы
                                       //                     (проверяются файлы description.json)

Перем НакопленныеДанные;               // Массив(Структура)      - результаты обработки данных

#Область ПрограммныйИнтерфейс

// Функция - признак возможности обработки, принимать входящие данные
// 
// Возвращаемое значение:
//	Булево - Истина - обработка может принимать входящие данные для обработки;
//	         Ложь - обработка не принимает входящие данные;
//
Функция ПринимаетДанные() Экспорт
	
	Возврат Ложь;
	
КонецФункции // ПринимаетДанные()

// Функция - признак возможности обработки, возвращать обработанные данные
// 
// Возвращаемое значение:
//	Булево - Истина - обработка может возвращать обработанные данные;
//	         Ложь - обработка не возвращает данные;
//
Функция ВозвращаетДанные() Экспорт
	
	Возврат Истина;
	
КонецФункции // ВозвращаетДанные()

// Функция - возвращает список параметров обработки
// 
// Возвращаемое значение:
//	Структура                                - структура входящих параметров обработки
//      *Тип                    - Строка         - тип параметра
//      *Обязательный           - Булево         - Истина - параметр обязателен
//      *ЗначениеПоУмолчанию    - Произвольный   - значение параметра по умолчанию
//      *Описание               - Строка         - описание параметра
//
Функция ОписаниеПараметров() Экспорт
	
	Параметры = Новый Структура();
	
	ДобавитьОписаниеПараметра(Параметры,
	                          "ПутьККаталогуДистрибутивов",
	                          "Строка",
	                          Истина,
	                          "",
	                          "путь к каталогу содержащему дистрибутивы конфигурации
	                          |для распаковки");

	ДобавитьОписаниеПараметра(Параметры,
	                          "КаталогДляРаспаковкиШаблонов",
	                          "Строка",
	                          Истина,
	                          "",
	                          "каталог для распаковки шаблонов конфигурации");

	ДобавитьОписаниеПараметра(Параметры,
	                          "КаталогВАрхивеДляРаспаковкиEFD",
	                          "Строка",
	                          Ложь,
	                          "",
	                          "регулярное выражение, отбора каталогов
	                          |в архиве EFD для распаковки");

	ДобавитьОписаниеПараметра(Параметры,
	                          "ФайлыДляРаспаковкиEFD",
	                          "Массив",
	                          Ложь,
	                          "",
	                          "список файлов для распаковки из архива EFD дистрибутива конфигурации,
	                          |если не указан, то распаковываются все файлы");

	ДобавитьОписаниеПараметра(Параметры,
	                          "РаспаковыватьСуществующие",
	                          "Булево",
	                          Ложь,
	                          Ложь,
	                          "Истина - будут распакованы все найденные дистрибутивы
	                          |независимо от существующих в каталоге для распаковки
	                          |Ложь - будут оаспакованы только отсутствующие
	                          |в каталоге для распаковки дистрибутивы
	                          |(проверяются файлы description.json)");

	Возврат Параметры;
	
КонецФункции // ОписаниеПараметров()

// Функция - Возвращает обработку - менеджер
// 
// Возвращаемое значение:
//	ВнешняяОбработкаОбъект - обработка-менеджер
//
Функция МенеджерОбработкиДанных() Экспорт
	
	Возврат МенеджерОбработкиДанных;
	
КонецФункции // МенеджерОбработкиДанных()

// Процедура - Устанавливает обработку - менеджер
//
// Параметры:
//	НовыйМенеджерОбработкиДанных      - ВнешняяОбработкаОбъект - обработка-менеджер
//
Процедура УстановитьМенеджерОбработкиДанных(Знач НовыйМенеджерОбработкиДанных) Экспорт
	
	МенеджерОбработкиДанных = НовыйМенеджерОбработкиДанных;
	
КонецПроцедуры // УстановитьМенеджерОбработкиДанных()

// Функция - Возвращает идентификатор обработчика
// 
// Возвращаемое значение:
//	Строка - идентификатор обработчика
//
Функция Идентификатор() Экспорт
	
	Возврат Идентификатор;
	
КонецФункции // Идентификатор()

// Процедура - Устанавливает идентификатор обработчика
//
// Параметры:
//	НовыйИдентификатор      - Строка - новый идентификатор обработчика
//
Процедура УстановитьИдентификатор(Знач НовыйИдентификатор) Экспорт
	
	Идентификатор = НовыйИдентификатор;
	
КонецПроцедуры // УстановитьИдентификатор()

// Функция - Возвращает значения параметров обработки
// 
// Возвращаемое значение:
//	Структура - параметры обработки
//
Функция ПараметрыОбработкиДанных() Экспорт
	
	Возврат ПараметрыОбработки;
	
КонецФункции // ПараметрыОбработкиДанных()

// Процедура - Устанавливает значения параметров обработки данных
//
// Параметры:
//	НовыеПараметры      - Структура     - значения параметров обработки
//
Процедура УстановитьПараметрыОбработкиДанных(Знач НовыеПараметры) Экспорт
	
	ПараметрыОбработки = НовыеПараметры;
	
	УстановитьПараметрОбработкиДанныхИзСтруктуры("ПутьККаталогуДистрибутивов"     , ПараметрыОбработки);
	УстановитьПараметрОбработкиДанныхИзСтруктуры("КаталогДляРаспаковкиШаблонов"   , ПараметрыОбработки);
	УстановитьПараметрОбработкиДанныхИзСтруктуры("КаталогВАрхивеДляРаспаковкиEFD" , ПараметрыОбработки);
	УстановитьПараметрОбработкиДанныхИзСтруктуры("ФайлыДляРаспаковкиEFD"          , ПараметрыОбработки);
	УстановитьПараметрОбработкиДанныхИзСтруктуры("РаспаковыватьСуществующие"      , ПараметрыОбработки);

КонецПроцедуры // УстановитьПараметрыОбработкиДанных()

// Функция - Возвращает значение параметра обработки данных
// 
// Параметры:
//	ИмяПараметра      - Строка           - имя получаемого параметра
//
// Возвращаемое значение:
//	Произвольный      - значение параметра
//
Функция ПараметрОбработкиДанных(Знач ИмяПараметра) Экспорт
	
	Если НЕ ТипЗнч(ПараметрыОбработки) = Тип("Структура") Тогда
		Возврат Неопределено;
	КонецЕсли;
	
	Если НЕ ПараметрыОбработки.Свойство(ИмяПараметра) Тогда
		Возврат Неопределено;
	КонецЕсли;
	
	Возврат ПараметрыОбработки[ИмяПараметра];
	
КонецФункции // ПараметрОбработкиДанных()

// Процедура - Устанавливает значение параметра обработки
//
// Параметры:
//	ИмяПараметра      - Строка           - имя устанавливаемого параметра
//	Значение          - Произвольный     - новое значение параметра
//
Процедура УстановитьПараметрОбработкиДанных(Знач ИмяПараметра, Знач Значение) Экспорт
	
	Если НЕ ТипЗнч(ПараметрыОбработки) = Тип("Структура") Тогда
		ПараметрыОбработки = Новый Структура();
	КонецЕсли;
	
	ПараметрыОбработки.Вставить(ИмяПараметра, Значение);

	Если НЕ ЕстьПеременнаяМодуля(ИмяПараметра) Тогда
		Возврат;
	КонецЕсли;

	Если ВРег(ИмяПараметра) = "ФАЙЛЫДЛЯРАСПАКОВКИEFD" Тогда
		Если ТипЗнч(Значение) = Тип("Массив") Тогда
			ФайлыДляРаспаковкиEFD = Значение;
		Иначе
			ФайлыДляРаспаковкиEFD = СтрРазделить(Значение, "|", Ложь);
		КонецЕсли;
	ИначеЕсли ВРег(ИмяПараметра) = "ПУТЬККАТАЛОГУДИСТРИБУТИВОВ" Тогда
		ВремФайл = Новый Файл(Значение);
		ПутьККаталогуДистрибутивов = ВремФайл.ПолноеИмя;
	ИначеЕсли ВРег(ИмяПараметра) = "КАТАЛОГДЛЯРАСПАКОВКИШАБЛОНОВ" Тогда
		ВремФайл = Новый Файл(Значение);
		КаталогДляРаспаковкиШаблонов = ВремФайл.ПолноеИмя;
	Иначе
		Выполнить(СтрШаблон("%1 = Значение;", ИмяПараметра));
	КонецЕсли;
	
КонецПроцедуры // УстановитьПараметрОбработкиДанных()

// Процедура - устанавливает данные для обработки
//
// Параметры:
//	ВходящиеДанные      - Структура     - значения параметров обработки
//
Процедура УстановитьДанные(Знач ВходящиеДанные) Экспорт
	
КонецПроцедуры // УстановитьДанные()

// Процедура - выполняет обработку данных
//
Процедура ОбработатьДанные() Экспорт

	ФайлыОписанийВерсий = НайтиФайлы(ПутьККаталогуДистрибутивов, "description.json", Истина);

	Если ФайлыОписанийВерсий.Количество() = 0 Тогда
		Лог.Ошибка("[%1]: Не найдены файлы описания версий ""description.json"" в каталоге %2,
		           |возможно каталог указан некорректно.",
		           ТипЗнч(ЭтотОбъект),
				   ПутьККаталогуДистрибутивов);
		Возврат;
	КонецЕсли;

	ВерсииДляОбработки = Новый Массив();

	Для Каждого ТекФайл Из ФайлыОписанийВерсий Цикл

		ОписаниеВерсии = Служебный.ОписаниеРелиза(ТекФайл.ПолноеИмя);

		ФайлАрхиваEFD = Новый Файл(ОбъединитьПути(ТекФайл.Путь, "1cv8.efd"));

		Если НЕ ФайлАрхиваEFD.Существует() Тогда
			Продолжить;
		КонецЕсли;

		ОписаниеВерсии.Вставить("КаталогВерсии", ТекФайл.Путь);

		ВерсииДляОбработки.Добавить(ОписаниеВерсии);
	КонецЦикла;

	Служебный.СортироватьОписанияВерсийПоДате(ВерсииДляОбработки);

	Для Каждого ТекОписание Из ВерсииДляОбработки Цикл

		Лог.Информация("[%1]: Обработка версии %2 (%3) конфигурации %4 из дистрибутива %5",
		               ТипЗнч(ЭтотОбъект),
		               ТекОписание.Версия,
		               Формат(ТекОписание.Дата, "ДФ=dd.MM.yyyy; ДП=-"),
		               ТекОписание.Имя,
		               ТекОписание.КаталогВерсии);

		ОтносительныйКаталогВерсии = СтрЗаменить(ТекОписание.КаталогВерсии, ПутьККаталогуДистрибутивов, "");
		Если Лев(ОтносительныйКаталогВерсии, 1) = "\" ИЛИ Лев(ОтносительныйКаталогВерсии, 1) = "/" Тогда
			ОтносительныйКаталогВерсии = Сред(ОтносительныйКаталогВерсии, 2);
		КонецЕсли;

		КаталогДляРаспаковкиEFD = ОбъединитьПути(КаталогДляРаспаковкиШаблонов, ОтносительныйКаталогВерсии);

		РаспаковщикРелиза = Новый РаспаковщикРелизов1С();
		РаспаковщикРелиза.УстановитьПараметрыОбработкиДанных(ПараметрыОбработки);
		РаспаковщикРелиза.УстановитьПараметрОбработкиДанных("Приложение_Имя"                 , ТекОписание.Имя);
		РаспаковщикРелиза.УстановитьПараметрОбработкиДанных("Приложение_Ид"                  , ТекОписание.Идентификатор);
		РаспаковщикРелиза.УстановитьПараметрОбработкиДанных("Приложение_Версия"              , ТекОписание.Версия);
		РаспаковщикРелиза.УстановитьПараметрОбработкиДанных("ПутьКДистрибутиву"              , ТекОписание.КаталогВерсии);
		РаспаковщикРелиза.УстановитьПараметрОбработкиДанных("КаталогДляРаспаковкиEFD"        , КаталогДляРаспаковкиEFD);
		РаспаковщикРелиза.УстановитьПараметрОбработкиДанных("КаталогВАрхивеДляРаспаковкиEFD" , КаталогВАрхивеДляРаспаковкиEFD);
		РаспаковщикРелиза.УстановитьПараметрОбработкиДанных("ФайлыДляРаспаковкиEFD"          , ФайлыДляРаспаковкиEFD);
		РаспаковщикРелиза.УстановитьПараметрОбработкиДанных("РаспаковыватьСуществующие"      , РаспаковыватьСуществующие);
		РаспаковщикРелиза.ОбработатьДанные();
		
		ПродолжениеОбработкиДанныхВызовМенеджера(ТекОписание);
	КонецЦикла;

	ЗавершениеОбработкиДанныхВызовМенеджера();

КонецПроцедуры // ОбработатьДанные()

Функция РезультатОбработки() Экспорт
	
	Возврат НакопленныеДанные;
	
КонецФункции // РезультатОбработки()

// Процедура - выполняет действия при окончании обработки данных
// и оповещает обработку-менеджер о завершении обработки данных
//
Процедура ЗавершениеОбработкиДанных() Экспорт
	
	Лог.Информация("[%1]: Завершение обработки данных.", ТипЗнч(ЭтотОбъект));

	ЗавершениеОбработкиДанныхВызовМенеджера();
	
КонецПроцедуры // ЗавершениеОбработкиДанных()

#КонецОбласти // ПрограммныйИнтерфейс

#Область СлужебныйПрограммныйИнтерфейс

// Функция - возвращает объект управления логированием
//
// Возвращаемое значение:
//  Объект      - объект управления логированием
//
Функция Лог() Экспорт
	
	Возврат Лог;

КонецФункции // Лог()

// Процедура - устанавливает описание команды
//
// Параметры:
//  Команда    - КомандаПриложения     - объект описание команды
//
Процедура ОписаниеКоманды(Команда) Экспорт
	
	Команда.Опция("p path", "", "путь к каталогу содержащему дистрибутивы конфигурации
	                            |для распаковки")
	       .ТСтрока()
	       .ВОкружении("YARD_DISTRIBS_PATH");

	Команда.Опция("ep extract-path", "", "каталог для распаковки шаблонов конфигурации")
	       .ТСтрока()
	       .ВОкружении("YARD_EXTRACT_PATH");

	Команда.Опция("ed extract-dir", "", "регулярное выражение, отбора каталогов
	                                    |в архиве EFD для распаковки")
	       .ТСтрока()
	       .ВОкружении("YARD_EXTRACT_DIR");

	Команда.Опция("ef extract-files", "", "список файлов для распаковки из архива дистрибутива, разделенный ""|"",
	                                      |если не указан, то распаковываются все файлы")
	       .ТСтрока()
	       .ВОкружении("YARD_EXTRACT_FILES");

	Команда.Опция("ee extract-existing", Ложь, "Истина - будут распакованы все найденные дистрибутивы
	                                           |независимо от существующих в каталоге для распаковки
	                                           |Ложь - будут оаспакованы только отсутствующие
	                                           |в каталоге для распаковки дистрибутивы
	                                           |(проверяются файлы description.json)")
	       .Флаг();

КонецПроцедуры // ОписаниеКоманды()

// Процедура - запускает выполнение команды устанавливает описание команды
//
// Параметры:
//  Команда    - КомандаПриложения     - объект  описание команды
//
Процедура ВыполнитьКоманду(Знач Команда) Экспорт

	ВыводОтладочнойИнформации = Команда.ЗначениеОпции("verbose");

	ПараметрыПриложения.УстановитьРежимОтладки(ВыводОтладочнойИнформации);

	УстановитьПараметрОбработкиДанных("ПутьККаталогуДистрибутивов"     , Команда.ЗначениеОпции("path"));
	УстановитьПараметрОбработкиДанных("КаталогДляРаспаковкиШаблонов"   , Команда.ЗначениеОпции("extract-path"));
	УстановитьПараметрОбработкиДанных("КаталогВАрхивеДляРаспаковкиEFD" , Команда.ЗначениеОпции("extract-dir"));
	
	ВремФайлы = СтрРазделить(Команда.ЗначениеОпции("extract-files"), "|", Ложь);
	УстановитьПараметрОбработкиДанных("ФайлыДляРаспаковкиEFD"     , ВремФайлы);
	УстановитьПараметрОбработкиДанных("РаспаковыватьСуществующие" , Команда.ЗначениеОпции("extract-existing"));

	ОбработатьДанные();

КонецПроцедуры // ВыполнитьКоманду()

#КонецОбласти // СлужебныйПрограммныйИнтерфейс

#Область СлужебныеПроцедурыВызоваМенеджераОбработкиДанных

// Процедура - выполняет действия обработки элемента данных
// и оповещает обработку-менеджер о продолжении обработки элемента
//
//	Параметры:
//		Элемент    - Произвольный     - Элемент данных для продолжения обработки
//
Процедура ПродолжениеОбработкиДанныхВызовМенеджера(Элемент)
	
	Если МенеджерОбработкиДанных = Неопределено Тогда
		Возврат;
	КонецЕсли;
	
	МенеджерОбработкиДанных.ПродолжениеОбработкиДанных(Элемент, Идентификатор);
	
КонецПроцедуры // ПродолжениеОбработкиДанныхВызовМенеджера()

// Процедура - выполняет действия при окончании обработки данных
// и оповещает обработку-менеджер о завершении обработки данных
//
Процедура ЗавершениеОбработкиДанныхВызовМенеджера()
	
	Если МенеджерОбработкиДанных = Неопределено Тогда
		Возврат;
	КонецЕсли;
	
	МенеджерОбработкиДанных.ЗавершениеОбработкиДанных(Идентификатор);
	
КонецПроцедуры // ЗавершениеОбработкиДанныхВызовМенеджера()

#КонецОбласти // СлужебныеПроцедурыВызоваМенеджераОбработкиДанных

#Область СлужебныеПроцедурыИФункции

// Процедура - добавляет описание параметра обработки
// 
// Параметры:
//     ОписаниеПараметров     - Структура      - структура описаний параметров
//     Параметр               - Строка         - имя параметра
//     Тип                    - Строка         - список возможных типов параметра
//     Обязательный           - Булево         - Истина - параметр обязателен
//     ЗначениеПоУмолчанию    - Произвольный   - значение параметра по умолчанию
//     Описание               - Строка         - описание параметра
//
Процедура ДобавитьОписаниеПараметра(ОписаниеПараметров
	                              , Параметр
	                              , Тип
	                              , Обязательный = Ложь
	                              , ЗначениеПоУмолчанию = Неопределено
	                              , Описание = "")
	
	Если НЕ ТипЗнч(ОписаниеПараметров) = Тип("Структура") Тогда
		ОписаниеПараметров = Новый Структура();
	КонецЕсли;
	
	ОписаниеПараметра = Новый Структура();
	ОписаниеПараметра.Вставить("Тип"                , Тип);
	ОписаниеПараметра.Вставить("Обязательный"       , Обязательный);
	ОписаниеПараметра.Вставить("ЗначениеПоУмолчанию", ЗначениеПоУмолчанию);
	ОписаниеПараметра.Вставить("Описание"           , Описание);
	
	ОписаниеПараметров.Вставить(Параметр, ОписаниеПараметра);
	
КонецПроцедуры // ДобавитьОписаниеПараметра()

// Процедура - устанавливает значение переменной модуля с указанным именем
// из значения структуры с тем же именем или значение по умолчанию
// 
// Параметры:
//	ИмяПараметра          - Строка           - имя параметра для установки значения
//	СтруктураПараметров   - Структура        - структуры значений параметров
//	ЗначениеПоУмолчанию   - Произвольный     - значение переменной по умолчанию
//
Процедура УстановитьПараметрОбработкиДанныхИзСтруктуры(Знач ИмяПараметра,
	                                                  Знач СтруктураПараметров,
	                                                  Знач ЗначениеПоУмолчанию = "")

	Если НЕ ЕстьПеременнаяМодуля(ИмяПараметра) Тогда
		Возврат;
	КонецЕсли;

	ЗначениеПараметра = ЗначениеПоУмолчанию;

	Если СтруктураПараметров.Свойство(ИмяПараметра) Тогда
		ЗначениеПараметра = СтруктураПараметров[ИмяПараметра];
	КонецЕсли;

	Выполнить(СтрШаблон("%1 = ЗначениеПараметра;", ИмяПараметра));

КонецПроцедуры // УстановитьПараметрОбработкиДанныхИзСтруктуры()

// Функция - проверяет наличие в текущем модуле переменной с указанным именем
// 
// Параметры:
//	ИмяПеременной      - Строка           - имя переменной для проверки
//
// Возвращаемое значение:
//	Булево      - Истина - переменная существует; Ложь - в противном случае.
//
Функция ЕстьПеременнаяМодуля(Знач ИмяПеременной)

	Попытка
		ЗначениеПеременной = Вычислить(ИмяПеременной);
	Исключение
		Возврат Ложь;
	КонецПопытки;

	Возврат Истина;

КонецФункции // ЕстьПеременнаяМодуля()

#КонецОбласти // СлужебныеПроцедурыИФункции

#Область ОбработчикиСобытий

// Процедура - обработчик события "ПриСозданииОбъекта"
//
// Параметры:
//  Менеджер	 - МенеджерОбработкиДанных    - менеджер обработки данных - владелец
// 
// BSLLS:UnusedLocalMethod-off
Процедура ПриСозданииОбъекта(Менеджер = Неопределено)

	УстановитьМенеджерОбработкиДанных(Менеджер);

	Лог = ПараметрыПриложения.Лог();

	Лог.Информация("[%1]: Инициализирован обработчик", ТипЗнч(ЭтотОбъект));

КонецПроцедуры // ПриСозданииОбъекта()
// BSLLS:UnusedLocalMethod-on

#КонецОбласти // ОбработчикиСобытий
